BASE_URL = 'https://home.personalcapital.com'
API_ENDPOINT = BASE_URL + '/api'
STAPLE_CATEGORIES = [
    # 'Personal Care',
    'Gasoline/Fuel',
    'Groceries',
    'Utilities',
    'Healthcare/Medical',
    'Rent',
    'Pets/Pet Care',
    'Education',
    'Cable/Satellite',
    'Insurance',
    'Loans',
    'Wedding'
]
